@@ -14,9 +14,16 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+import sys
 import ldap
 from utils.common import _error
 from collections import defaultdict
+from ldif import LDIFParser, LDIFWriter
+import cStringIO
+
+from calculate.lib.cl_lang import setLocalTranslate
+_ = lambda x: x
+setLocalTranslate('cl_lib3', sys.modules[__name__])
 
 
 class ldapFun(_error):
@@ -61,6 +68,142 @@ class ldapFun(_error):
         return result_set
 
 
+class LDAPConnectError(Exception):
+    pass
+
+class LDAPBadSearchFilter(LDAPConnectError):
+    pass
+
+class LDAPConnect(ldapFun):
+    """
+    Объект работающий с исключениями
+    """
+
+    def setError(self, message):
+        raise LDAPConnectError(message)
+
+    def ldap_search(self, base_dn, search_scope=ldap.SCOPE_BASE,
+                    search_filter='(objectClass=*)',
+                    retrieve_attrs=None):
+        try:
+            ldap_result_id = self.conLdap.search(
+                base_dn, search_scope, search_filter, retrieve_attrs)
+            while 1:
+                result_type, result_data = self.conLdap.result(
+                    ldap_result_id, 0)
+                if isinstance(result_data, list) and not result_data:
+                    break
+                else:
+                    if result_type == ldap.RES_SEARCH_ENTRY:
+                        yield result_data
+        except ldap.NO_SUCH_OBJECT:
+            pass
+        except ldap.LDAPError, e:
+            error = e[0]['desc']
+            if "Bad search filter" in error:
+                raise LDAPBadSearchFilter(error)
+            raise LDAPConnectError(error)
+        except Exception as e:
+            raise LDAPConnectError(str(e))
+
+    def ldap_dump(self, fobj, base_dn, search_scope=ldap.SCOPE_SUBTREE,
+                  search_filter='(objectClass=*)', retrieve_attrs=None):
+        """
+        вернуть dump ldif
+        :param fobj:
+        :param base_dn:
+        :param search_scope:
+        :param search_filter:
+        :param retrieve_attrs:
+        :return:
+        """
+        try:
+            dn_list = self.conLdap.search_s(
+                base_dn, search_scope, search_filter, retrieve_attrs)
+            writer = LDIFWriter(fobj)
+            for dn, f in dn_list:
+                writer.unparse(dn, f)
+        except ldap.NO_SUCH_OBJECT:
+            pass
+        except ldap.LDAPError, e:
+            error = e[0]['desc']
+            if "Bad search filter" in error:
+                raise LDAPBadSearchFilter(error)
+            raise LDAPConnectError(error)
+        except Exception as e:
+            raise LDAPConnectError(str(e))
+
+    def ldap_simple_search(self, base_dn, search_filter, attr, first=True):
+        """
+        Простой поиск аттрибутов
+        :param base_dn:
+        :param search_filter:
+        :param attr:
+        :param first: вернуть первый из списка
+        :return:
+        """
+        for entry in self.ldap_search(base_dn, ldap.SCOPE_ONELEVEL,
+                                      search_filter, [attr]):
+            attrs = entry[0][1]
+            yield attrs[attr][0]
+
+    def ldap_modify_attrs(self, base_dn, attrs):
+        """Модифицирует аттрибуты DN"""
+        attrs = list(attrs)
+        if attrs:
+            try:
+                self.conLdap.modify_s(base_dn, attrs)
+            except ldap.LDAPError, e:
+                raise LDAPConnectError(e[0]['desc'])
+
+    def ldap_modify_dn(self, base_dn, new_dn):
+        """
+        Изменить ветку
+        :param base_dn: предыдущее название
+        :param new_dn: новое название
+        :return:
+        """
+        try:
+            self.conLdap.modrdn_s(base_dn, new_dn)
+        except ldap.LDAPError, e:
+            raise LDAPConnectError(e[0]['desc'])
+
+    def ldap_remove_dn(self, base_dn):
+        """
+        Удалить указанный dn
+        :param base_dn: удаляемый dn
+        :return:
+        """
+        try:
+            self.conLdap.delete_s(base_dn)
+        except ldap.LDAPError, e:
+            raise LDAPConnectError(e[0]['desc'])
+
+
+
+            # @adminConnectLdap
+            # def modifyElemDN(self, relDN, newFirstDn):
+            #    """Изменяет основной элемент DN (uid, cn и др.)"""
+            #    DN = self.addDN(relDN,self.baseDN)
+            #    try:
+            #        self.conLdap.modrdn_s(DN, newFirstDn)
+            #    except ldap.LDAPError, e:
+            #        self.printERROR(e[0]['desc'])
+            #        return False
+            #    return True
+
+            # @adminConnectLdap
+            # def delDN(self, relDN):
+            #    """Удаляет одиночный DN"""
+            #    DN = self.addDN(relDN,self.baseDN)
+            #    try:
+            #        self.conLdap.delete_s(DN)
+            #    except ldap.LDAPError, e:
+            #        self.printERROR(e[0]['desc'])
+            #        return False
+            #    return True
+
+
 class ldapUser(_error):
     """Получение данных для пользователя из LDAP"""
     # Данные из /etc/ldap.conf
@@ -252,3 +395,27 @@ class ldapUser(_error):
             self.ldapObj = ldap_obj
             self.conLdap = ldap_obj.conLdap
         return True
+
+
+class LDIFError(Exception):
+    pass
+
+
+class LDIFAdd(LDIFParser):
+    """
+    Добавление LDIF в базу
+    """
+
+    def __init__(self, ldif_data, ldap_connect):
+        self.ldap_connect = ldap_connect
+        LDIFParser.__init__(self, cStringIO.StringIO(ldif_data))
+
+    def handle(self, dn, entry, controls=None):
+        # (self, dn, entry, *args):
+        try:
+            self.ldap_connect.add_s(dn, entry.items())
+        except ldap.LDAPError as e:
+            raise LDIFError(e[0]['desc'])
+        except Exception as e:
+            print str(e)
+            raise LDIFError(_("Error in LDIF file"))
