@@ -18,6 +18,8 @@ import sys
 from calculate.lib.cl_template import TemplateFormat
 from calculate.lib.utils.text import _u
 from calculate.lib.cl_lang import setLocalTranslate
+from os import path
+import re
 
 _ = lambda x: x
 setLocalTranslate('cl_lib3', sys.modules[__name__])
@@ -55,9 +57,16 @@ class world(TemplateFormat):
     def textToXML(self):
         """Создание документа из текста self.text
         """
+        category = path.basename(self.template_name)
+
+        def autocategory(name):
+            if name and "/" not in name:
+                prefix, op, name = name.rpartition("!")
+                return "%s%s%s/%s" % (prefix, op, category, name)
+            return name
         try:
             doc = filter(None,
-                         [x.strip() for x in self.text.split("\n")
+                         [autocategory(x.strip()) for x in self.text.split("\n")
                           if not x.startswith("#")])
         except ValueError:
             doc = False
@@ -82,8 +91,29 @@ class world(TemplateFormat):
         for line in new_doc:
             if line.startswith("!"):
                 pkg = line[1:]
-                if pkg in old_doc:
-                    old_doc.remove(pkg)
+                # если удаление с проверкой и предупреждением
+                if pkg.startswith("!"):
+                    warn = True
+                    pkg = pkg[1:]
+                else:
+                    warn = False
+                if pkg.endswith(":*"):
+                    re_match = re.compile("^{0}$|^{0}:.*$".format(pkg[:-2])).match
+                    remove_records = [x for x in old_doc if re_match(x)]
+                    if remove_records:
+                        for del_pkg in remove_records:
+                            old_doc.remove(del_pkg)
+                    else:
+                        warn &= True
+                else:
+                    if pkg in old_doc:
+                        old_doc.remove(pkg)
+                    else:
+                        warn &= True
+                if warn:
+                    if callable(self.parent.printWARNING):
+                        self.parent.printWARNING(
+                            _("Not found {package} in world").format(package=pkg))
             else:
                 if line not in old_doc:
                     old_doc.append(line)
