@@ -18,12 +18,15 @@ import os
 from os import path, R_OK
 import platform
 import re
+import errno
 from calculate.lib.utils.dracut import Dracut
 from calculate.lib.datavars import Variable, ReadonlyVariable
 from calculate.lib.utils.files import (readFile, process, listDirectory,
-                                       isMount, pathJoin, Mounts)
+                                       pathJoin)
+from calculate.lib.utils.mount import isMount, Mounts
 from calculate.lib.utils.system import SystemType
-from calculate.lib.utils.device import getUdevDeviceInfo, loadEfiVars
+import calculate.lib.utils.device as device
+from calculate.lib.utils.device import loadEfiVars
 from calculate.lib.utils.common import getKernelUid, getValueFromCmdLine, \
     CmdlineParams
 
@@ -66,7 +69,7 @@ class VariableOsRootDev(ReadonlyVariable):
             rootparam = re_res.group(1)
             # check root for /dev/sd view
             if re.match("^/dev/[a-z]+.*$", rootparam):
-                return getUdevDeviceInfo(
+                return device.udev.get_device_info(
                     name=rootparam.strip()).get('DEVNAME', rootparam)
             # check root set by uuid
             if re.match("^UUID=.*$", rootparam):
@@ -74,7 +77,7 @@ class VariableOsRootDev(ReadonlyVariable):
                 blkid_process = process('/sbin/blkid', '-c', '/dev/null', '-U',
                                         uuid)
                 if blkid_process.success():
-                    return getUdevDeviceInfo(
+                    return device.udev.get_device_info(
                         name=blkid_process.read().strip()).get('DEVNAME', '')
             # check root set by label
             if re.match("^LABEL=.*$", rootparam):
@@ -82,7 +85,7 @@ class VariableOsRootDev(ReadonlyVariable):
                 blkid_process = process('/sbin/blkid', '-c', '/dev/null', '-L',
                                         uuid)
                 if blkid_process.success():
-                    return getUdevDeviceInfo(
+                    return device.udev.get_device_info(
                         name=blkid_process.read().strip()).get('DEVNAME', '')
         # get device mounted to root
         df_process = process('df', '/', envdict={'LANG': 'C'})
@@ -106,7 +109,8 @@ class VariableOsMapperRootDev(ReadonlyVariable):
 
     def get(self):
         rootdev = self.Get('os_root_dev')
-        dev_links = getUdevDeviceInfo(name=rootdev).get("DEVLINKS", "")
+        dev_links = device.udev.get_device_info(
+            name=rootdev).get("DEVLINKS", "")
         if dev_links:
             mapper_name = filter(lambda x: "/dev/mapper" in x,
                                  dev_links.split())
@@ -297,9 +301,8 @@ class VariableOsUefiSet(ReadonlyVariable):
     """
 
     def get(self):
-        if not path.exists('/sys/firmware/efi'):
-            loadEfiVars()
-        if path.exists('/sys/firmware/efi'):
+        loadEfiVars()
+        if device.sysfs.exists(device.sysfs.Path.Efi):
             return "on"
         else:
             return "off"
@@ -342,3 +345,20 @@ class VariableOsSubsystem(Variable):
 
     def get(self):
         return SystemType.detect()
+
+
+class VariableClRootReadonly(ReadonlyVariable):
+    """
+    """
+    type = "bool"
+
+    def get(self):
+        check_fn = '/var/tmp/.calculate-rw-check-%d' % os.getpid()
+        try:
+            with open(check_fn,'w') as f:
+                pass
+            os.unlink(check_fn)
+        except (IOError,OSError) as e:
+            if e.errno == errno.EROFS:
+                return "on"
+        return "off"
